#include "exceptions.hh"
#include "lohi.hh"
#include "order.hh"
#include "driver_calc.hh"
#include "cmdline.hh"
#include "printout.hh"
#include <error.h>

namespace {
    void report_error_and_exit(const error_t code, const char *const message)
    { error(EXIT_FAILURE, code, "%s", message); }
}

int main(const int argc, char **const argv)
try
{
    Driver::Cmdline_args cmdline_args {Driver::read_cmdline(argc, argv)};
    Interval::Order order {cmdline_args.order_max};
    const Interval::Lohi result {
        Driver::calculate_by_name(
            &cmdline_args.precision,
            cmdline_args.name_of_number,
            cmdline_args.parameters,
            cmdline_args.prefer_negative_branch,
            &order
        )
    };
    Driver::printout(
        cmdline_args.precision, result,
        cmdline_args.facets_to_print, order.count()
    );
    return 0;
}
catch (const Interval::Division_by_zero)
{ report_error_and_exit(EDOM, "division by zero"); }
catch (const Interval::Forbidden_negative)
{ report_error_and_exit(EDOM, "forbidden negative"); }
catch (...)
{ report_error_and_exit(ENOTSUP, "unknown error"); }

