#include "../../src/gcd.hh"
#include <cassert>

int main()
{
    assert(Interval::gcd( 12,  30) ==  6);
    assert(Interval::gcd( 12, -30) == -6);
    assert(Interval::gcd(-12, -30) == -6);
    assert(Interval::gcd(-12,  30) ==  6);
    assert(Interval::gcd( 30,  12) ==  6);
    assert(Interval::gcd(-30,  12) ==  6);
    assert(Interval::gcd(-30, -12) == -6);
    assert(Interval::gcd( 30, -12) == -6);
    assert(Interval::gcd( 30,  13) ==  1);
    assert(Interval::gcd( 30,   1) ==  1);
    assert(Interval::gcd( 13,  30) ==  1);
    assert(Interval::gcd(  1,  30) ==  1);
    assert(Interval::gcd(  0,  30) == 30);
    {
        int has_thrown {0};
        try { [[maybe_unused]] long a {Interval::gcd( 0, 0)}; } catch (const Interval::Division_by_zero) { ++has_thrown; }
        try { [[maybe_unused]] long a {Interval::gcd(30, 0)}; } catch (const Interval::Division_by_zero) { ++has_thrown; }
        try { [[maybe_unused]] long a {Interval::gcd(-1, 0)}; } catch (const Interval::Division_by_zero) { ++has_thrown; }
        assert(has_thrown == 3);
    }
    { long a { 12}, b { 30}; const long g {Interval::reduce(&a, &b)}; assert(a ==  2 && b ==  5 && g ==  6); }
    { long a {-12}, b { 30}; const long g {Interval::reduce(&a, &b)}; assert(a == -2 && b ==  5 && g ==  6); }
    { long a {-12}, b {-30}; const long g {Interval::reduce(&a, &b)}; assert(a ==  2 && b ==  5 && g == -6); }
    { long a { 12}, b {-30}; const long g {Interval::reduce(&a, &b)}; assert(a == -2 && b ==  5 && g == -6); }
    { long a { 12}, b { 30}; const long g {Interval::reduce(&b, &a)}; assert(a ==  2 && b ==  5 && g ==  6); }
    { long a {-12}, b { 30}; const long g {Interval::reduce(&b, &a)}; assert(a ==  2 && b == -5 && g == -6); }
    { long a {-12}, b {-30}; const long g {Interval::reduce(&b, &a)}; assert(a ==  2 && b ==  5 && g == -6); }
    { long a { 12}, b {-30}; const long g {Interval::reduce(&b, &a)}; assert(a ==  2 && b == -5 && g ==  6); }
    { long a { 30}, b { 13}; const long g {Interval::reduce(&a, &b)}; assert(a == 30 && b == 13 && g ==  1); }
    { long a { 30}, b {  1}; const long g {Interval::reduce(&a, &b)}; assert(a == 30 && b ==  1 && g ==  1); }
    { long a {  1}, b { 30}; const long g {Interval::reduce(&a, &b)}; assert(a ==  1 && b == 30 && g ==  1); }
    { long a {  0}, b { 30}; const long g {Interval::reduce(&a, &b)}; assert(a ==  0 && b ==  1 && g == 30); }
    { long a { 30}, b {  1}; const long g {Interval::reduce(&a, &b)}; assert(a == 30 && b ==  1 && g ==  1); }
    {
        int has_thrown {0};
        long a, b;
        a =   0; b = 0; try { Interval::reduce(&a, &b); } catch (const Interval::Division_by_zero) { ++has_thrown; }
        a = -30; b = 0; try { Interval::reduce(&a, &b); } catch (const Interval::Division_by_zero) { ++has_thrown; }
        a =   1; b = 0; try { Interval::reduce(&a, &b); } catch (const Interval::Division_by_zero) { ++has_thrown; }
        assert(has_thrown == 3);
    }
    return 0;
}

